/**-----------------------------------------------------------------------------------------
* Copyright © 2025 Progress Software Corporation. All rights reserved.
* Licensed under commercial license. See LICENSE.md in the project root for more information
*-------------------------------------------------------------------------------------------*/
import { Directive, NgZone, Renderer2 } from '@angular/core';
import { ToolBarButtonComponent } from '@progress/kendo-angular-toolbar';
import { Subscription } from 'rxjs';
import { filterIcon } from '@progress/kendo-svg-icons';
import { PopupService } from '@progress/kendo-angular-popup';
import { ContextService } from '../../../common/provider.service';
import { closest, isDocumentAvailable, isPresent } from '@progress/kendo-angular-common';
import { ColumnInfoService } from '../../../common/column-info.service';
import { take } from 'rxjs/operators';
import { FilterService } from '../../../filtering/filter.service';
import { FilterToolWrapperComponent } from './filter-tool-wrapper.component';
import * as i0 from "@angular/core";
import * as i1 from "@progress/kendo-angular-toolbar";
import * as i2 from "@progress/kendo-angular-popup";
import * as i3 from "../../../common/provider.service";
import * as i4 from "../../../filtering/filter.service";
import * as i5 from "../../../common/column-info.service";
let incrementingId = 0;
/**
 * Represents the toolbar tool for filtering columns of the Grid.
 * You can apply this directive to any `kendo-toolbar-button` element inside a
 * ToolbarComponent used in the Grid.
 *
 * When the user clicks the toolbar button that is associated with the directive, the
 * filter event is triggered.
 *
 * @example
 * ```html-no-run
 * <kendo-grid>
 *      <kendo-toolbar>
 *         <kendo-toolbar-button text="Filter" kendoGridFilterTool></kendo-toolbar-button>
 *      </kendo-toolbar>
 * </kendo-grid>
 * ```
 */
export class FilterCommandToolbarDirective {
    host;
    popupService;
    ctx;
    filterService;
    columnInfoService;
    ngZone;
    renderer;
    popupRef;
    nextId = incrementingId++;
    toolSubs = new Subscription();
    popupSubs;
    removeClickListener;
    constructor(host, popupService, ctx, filterService, columnInfoService, ngZone, renderer) {
        this.host = host;
        this.popupService = popupService;
        this.ctx = ctx;
        this.filterService = filterService;
        this.columnInfoService = columnInfoService;
        this.ngZone = ngZone;
        this.renderer = renderer;
    }
    ngOnInit() {
        this.toolSubs = this.host.click.subscribe(e => this.onClick(e));
        this.toolSubs.add(this.filterService.changes.subscribe(filter => {
            this.host.showBadge = isPresent(filter) && filter.filters.length > 0;
        }));
        const hasToolbarIcon = isPresent(this.host.toolbarOptions.icon) && this.host.toolbarOptions.icon !== '';
        const hasOverflowIcon = isPresent(this.host.overflowOptions.icon) && this.host.overflowOptions.icon !== '';
        const hasIcon = hasToolbarIcon && hasOverflowIcon;
        const hasSvgIcon = isPresent(this.host.toolbarOptions.svgIcon) && isPresent(this.host.overflowOptions.svgIcon);
        if (!hasIcon) {
            this.host.icon = 'k-i-filter';
        }
        if (!hasSvgIcon) {
            this.host.svgIcon = filterIcon;
        }
    }
    ngAfterViewInit() {
        if (!isPresent(this.host.text)) {
            this.ngZone.onStable.pipe(take(1)).subscribe(() => {
                this.host.text = this.ctx.localization.get(`filterToolbarToolText`);
            });
        }
        this.host.toolbarButtonElement.nativeElement.setAttribute('aria-haspopup', 'dialog');
        this.host.toolbarButtonElement.nativeElement.setAttribute('aria-expanded', 'false');
        this.host.toolbarButtonElement.nativeElement.setAttribute('title', this.ctx.localization.get('filter'));
    }
    ngOnDestroy() {
        if (this.toolSubs) {
            this.toolSubs.unsubscribe();
        }
        if (this.popupSubs) {
            this.popupSubs.unsubscribe();
        }
        if (this.popupRef) {
            this.popupRef.close();
            this.popupRef = null;
        }
        if (this.removeClickListener) {
            this.removeClickListener();
            this.removeClickListener = null;
        }
    }
    onClick(e) {
        e.preventDefault();
        if (this.popupRef) {
            this.closePopup();
            return;
        }
        this.openPopup();
    }
    openPopup() {
        const direction = this.ctx.localization.rtl ? 'right' : 'left';
        this.popupRef = this.popupService.open({
            anchor: this.host.toolbarButtonElement.nativeElement,
            content: FilterToolWrapperComponent,
            popupClass: 'k-grid-columnmenu-popup',
            positionMode: 'absolute',
            anchorAlign: { vertical: 'bottom', horizontal: direction },
            popupAlign: { vertical: 'top', horizontal: direction }
        });
        this.setPopupAttributes();
        this.ngZone.runOutsideAngular(() => {
            if (!isDocumentAvailable()) {
                return;
            }
            this.removeClickListener = this.renderer.listen('document', 'click', (e) => {
                if (this.popupRef && !closest(e.target, node => node === this.popupRef.popupElement || node === this.host.toolbarButtonElement.nativeElement)) {
                    this.closePopup();
                }
            });
        });
        this.popupSubs = this.popupRef.popup.instance.anchorViewportLeave.subscribe(() => {
            this.popupSubs?.unsubscribe();
            this.popupSubs = null;
            this.closePopup();
        });
        this.initPopupProperties();
    }
    setPopupAttributes() {
        const popupElement = this.popupRef.popupElement;
        const popupId = `k-filter-tool-${this.nextId}-popup`;
        const popupAriaElement = popupElement.querySelector('.k-popup');
        this.renderer.setAttribute(popupElement, 'dir', this.ctx.localization.rtl ? 'rtl' : 'ltr');
        this.renderer.setAttribute(popupAriaElement, 'id', popupId);
        this.renderer.setAttribute(popupAriaElement, 'role', 'dialog');
        this.host.toolbarButtonElement.nativeElement.setAttribute('aria-expanded', 'true');
        this.host.toolbarButtonElement.nativeElement.setAttribute('aria-controls', popupId);
    }
    initPopupProperties() {
        this.popupRef.content.instance.columnInfoService = this.columnInfoService;
        this.popupRef.content.instance.ctx = this.ctx;
        this.popupRef.content.instance.filterService = this.filterService;
        this.popupRef.content.instance.hostButton = this.host;
        this.popupSubs.add(this.popupRef.content.instance.close.subscribe(() => {
            this.closePopup();
        }));
    }
    closePopup() {
        this.host.toolbarButtonElement.nativeElement.setAttribute('aria-expanded', 'false');
        this.host.toolbarButtonElement.nativeElement.removeAttribute('aria-controls');
        if (this.popupRef) {
            this.popupRef.close();
            this.popupRef = null;
        }
        if (this.popupSubs) {
            this.popupSubs.unsubscribe();
            this.popupSubs = null;
        }
        if (this.removeClickListener) {
            this.removeClickListener();
            this.removeClickListener = null;
        }
    }
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "16.2.12", ngImport: i0, type: FilterCommandToolbarDirective, deps: [{ token: i1.ToolBarButtonComponent }, { token: i2.PopupService }, { token: i3.ContextService }, { token: i4.FilterService }, { token: i5.ColumnInfoService }, { token: i0.NgZone }, { token: i0.Renderer2 }], target: i0.ɵɵFactoryTarget.Directive });
    static ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "14.0.0", version: "16.2.12", type: FilterCommandToolbarDirective, isStandalone: true, selector: "[kendoGridFilterTool]", ngImport: i0 });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "16.2.12", ngImport: i0, type: FilterCommandToolbarDirective, decorators: [{
            type: Directive,
            args: [{
                    selector: '[kendoGridFilterTool]',
                    standalone: true
                }]
        }], ctorParameters: function () { return [{ type: i1.ToolBarButtonComponent }, { type: i2.PopupService }, { type: i3.ContextService }, { type: i4.FilterService }, { type: i5.ColumnInfoService }, { type: i0.NgZone }, { type: i0.Renderer2 }]; } });
